/*
    Some Java trash for file selectors etc.

    Changelog:
    
    1.12.2020: No use setting a standard size for the buttons
    1.12.2020: Align button text left for better readability
    1.12.2020: Lifted the AWT fileselector from PETSCII
    1.12.2020: Lifted selector() from PETSCII as well
    1.12.2020: Lifted javatheme() from PETSCII -> more native look on Win/Lin
    1.12.2020: Moving toward P3 compatibility
    3.5.2016: Turned selector() vertical and improved centering
    2.5.2016: Added menuselector()
    28.4.2016: UIManager things belong rather here
    26.4.2016: The first version
*/

import java.awt.event.*;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.*;
import javax.swing.*;
import javax.swing.filechooser.FileNameExtensionFilter;
import java.nio.file.*;
import java.io.*;

void javatheme()
{
    if(prefs.awtselector==-1) // Depending on the platform use either Swing or AWT fileselector (unless overridden in prefs)
        switch(platform)
        {
            case LINUX: prefs.awtselector=1; break; 
            case WINDOWS: prefs.awtselector=0; break; 
            case MACOSX: prefs.awtselector=1; break;
            default: prefs.awtselector=0; break;
        }
    
    try // Let's try to set a native look
    {
        if(platform==LINUX)
        {
            UIManager.setLookAndFeel("com.sun.java.swing.plaf.gtk.GTKLookAndFeel");
        }
        else
        {
            if(platform==WINDOWS)
            {
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
            }
            else // Others or Mac, better go with something generic (too bad)
            {
                UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
            }
        }
    }
    catch (Exception e) {};
}

class Selector extends JPanel implements ActionListener
{
    int selection;
    protected JButton b[];
    
    Selector(String title,String s) // Split string into options
    {
        Box box=Box.createVerticalBox();
        
        String splitz[]=splitTokens(s,",");
        b=new JButton[splitz.length];
        
        setLayout(new GridLayout(splitz.length+1,1,1,1));
        
        add(new JLabel(title,SwingConstants.CENTER));
        
        for(int i=0;i<splitz.length;i++)
        {
            b[i]=new JButton(splitz[i]);
            b[i].setActionCommand(str(i+'0'));
            b[i].addActionListener(this);
            //b[i].setPreferredSize(new Dimension(140,26));
            b[i].setHorizontalAlignment(SwingConstants.LEFT);
            
            add(b[i]);
        }
        
        selection=-1;
    }
    public void actionPerformed(ActionEvent e)
    {
        for(int i=0;i<b.length;i++)
        {
            if(str(i+'0').equals(e.getActionCommand()))
                selection=i;
        }
    }
}

// Select from a list
int selector(String title,String opt)
{
    JFrame frame=new JFrame("");
    //frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    frame.setLocationRelativeTo(null);
    
    Selector s=new Selector(title,opt);
    s.setOpaque(true);
    frame.setContentPane(s);
 
    //Display the window.
    frame.pack();
    frame.setVisible(true);
    frame.setLocationRelativeTo(null); // About in the middle
    
    while(s.selection==-1)
    {
        frame.toFront(); // A bit of a kludge to force the window on top
        frame.repaint();
        try { Thread.sleep(200); }
        catch(Exception e){};
    }
    
    frame.setVisible(false);
    
    System.gc(); // It'll leak anyway...
    return(s.selection);
}

// Select an option from a list
int menuselector(String title,String menutitle,String choices[])
{ 
    String input=(String)JOptionPane.showInputDialog(null,menutitle,title,JOptionPane.QUESTION_MESSAGE,
                                                     null,choices,choices[0]);
                                           
    if(input==null)
        return -1;
    else
    {
        for(int i=0;i<choices.length;i++)
            if(input.equals(choices[i]))
                return i;
    }
    return -1;
}

final int LOAD=0,
          SAVE=1;

// File selector
String fileselector(String dir,String filtername,String filter[],int mode)
{
    if(prefs.awtselector==1)
    {
        FileDialog fd;
        if(mode==SAVE)
            fd=new FileDialog(frame, "Select a file", FileDialog.SAVE);
        else
            fd=new FileDialog(frame, "Select a file", FileDialog.LOAD);

        fd.setDirectory(dir);
        fd.setFilenameFilter(new Filsu(filter));
    
        delay(100); // Helps with clicks?

        fd.setAlwaysOnTop(true);
        fd.setSize(800,600);
        fd.setLocationRelativeTo(null);
        fd.pack();
        fd.toFront();
        fd.requestFocus();     
        fd.setVisible(true); // Show it
        
        // Trying to get the window back to focus after selection, but this is just guessing
        surface.setVisible(true);
        frame.toFront();
        frame.requestFocus();
        
        if(fd.getDirectory()==null || fd.getFile()==null)
            return null;

        prefs.path=fd.getDirectory();

        return fd.getDirectory()+fd.getFile();
    }
    else
    {
        JFileChooser fc=new JFileChooser(dir);
        
        fc.setPreferredSize(new Dimension(480, 500));
        fc.setDialogTitle("Select a File");
        
        fc.setFileFilter(new FileNameExtensionFilter(filtername,filter));
        
        if(mode==LOAD)
            fc.setApproveButtonText("Load");
        else
            fc.setApproveButtonText("Save");
        
        int returnVal = fc.showOpenDialog(null);
        
        if(returnVal==JFileChooser.APPROVE_OPTION)
        {
            // Save cwd for next time
            prefs.path=fc.getCurrentDirectory().getPath();
            
            File file = fc.getSelectedFile();
            return file.getPath();
        }
        else
            return null;
    }
}

class Filsu implements FilenameFilter // Had to hack something like this for the AWT FileDialog
{
    String patt[];
    
    Filsu(String s[])
    {
        patt=s;
    }   
    boolean accept(File dir,String name)
    {
        for(int i=0;i<patt.length;i++)
            if(name.toLowerCase().endsWith(patt[i]))
                return true;
            
        return false;
    }
}
